-------------------------------------------------------------------------------
-- Title      : FFPG core wrapper
-- Project    : FMC DEL 1ns 2cha (FFPG)
-- URL        : http://www.ohwr.org/projects/fmc-del-1ns-2cha
-------------------------------------------------------------------------------
-- File       : FfpgCoreWrapper.vhd
-- Author(s)  : Jan Pospisil <j.pospisil@cern.ch>
-- Company    : CERN (BE-BI-QP)
-- Created    : 2016-06-28
-- Last update: 2016-08-24
-- Standard   : VHDL'93/02
-------------------------------------------------------------------------------
-- Description: Wrapper of the FFPG core with platform specifc primitives.
-------------------------------------------------------------------------------
-- Copyright (c) 2016 CERN (BE-BI-QP)
-------------------------------------------------------------------------------
-- GNU LESSER GENERAL PUBLIC LICENSE
-------------------------------------------------------------------------------
-- This source file is free software; you can redistribute it and/or modify it
-- under the terms of the GNU Lesser General Public License as published by the
-- Free Software Foundation; either version 2.1 of the License, or (at your
-- option) any later version. This source is distributed in the hope that it
-- will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- See the GNU Lesser General Public License for more details. You should have
-- received a copy of the GNU Lesser General Public License along with this
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author
-- 2016-08-24  1.0      Jan Pospisil
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library unisim;
use unisim.vcomponents.all;

library work;
use work.FfpgPkg.all;
use work.wishbone_pkg.all;

entity FfpgCoreWrapper is
    generic (
        g_ClkFrequency: positive; -- input clock frequency in Hz
        g_Version: std_logic_vector(31 downto 0)
    );
    port (
        -- Wishbone connection
        Clk_ik: in std_logic;
        Reset_ira: in std_logic; -- synchronized locally for better timing
        Wb_i: in t_wishbone_slave_in;
        Wb_o: out t_wishbone_slave_out;
        --- FMC interface
        -- clock
        ClkIn0P_ik: in std_logic; -- 1 MHz <= f_ClkIn0_ik <= 204 MHz
        ClkIn0N_ik: in std_logic; -- 1 MHz <= f_ClkIn0_ik <= 204 MHz
        -- DACs
        TriggerDac_o: out t_Ad5600Interface;
        VcxoDac_o: out t_Ad5600Interface;
        -- output enable
        Ch1OutputEnable_o: out std_logic;
        Ch2OutputEnable_o: out std_logic;
        -- delay configuration
        DelayValue_ob: out unsigned(9 downto 0);
        Ch1SetLe_on: out std_logic;
        Ch1ResLe_on: out std_logic;
        Ch2SetLe_on: out std_logic;
        Ch2ResLe_on: out std_logic;
        --
        Ch1SetP_o: out std_logic;
        Ch1SetN_o: out std_logic;
        Ch1ResP_o: out std_logic;
        Ch1ResN_o: out std_logic;
        Ch2SetP_o: out std_logic;
        Ch2SetN_o: out std_logic;
        Ch2ResP_o: out std_logic;
        Ch2ResN_o: out std_logic;
        TriggerP_i: in std_logic;
        TriggerN_i: in std_logic;
        -- AD9512 SPI
        SpiAd9512Sclk_o: out std_logic;
        SpiAd9512Mosi_o: out std_logic;
        SpiAd9512Miso_i: in std_logic;
        SpiAd9512Cs_on: out std_logic;
        -- AD9512 func. pin
        Ad9512Func_o: out std_logic;
        -- clock selection pin of SY58017U
        Clk2Sel_o: out std_logic;
        -- clock output
        ClkOutP_ok: out std_logic;
        ClkOutN_ok: out std_logic;
        -- temperature chip interface
        Onewire_io: inout std_logic;
        -- LEDs
        Led_ob: out std_logic_vector(4 downto 1)
    );
end entity;

architecture sp6 of FfpgCoreWrapper is

    -- for differential inputs
    signal ClkIn0_k, Trigger: std_logic;

    -- for differentials outputs
    signal Ch1Set, Ch1Res, Ch2Set, Ch2Res: std_logic;
    signal ClkOutDdr, ClkOut_k: std_logic;

begin

    cClkIn0Ibufgds: IBUFGDS
        generic map (
            DIFF_TERM => TRUE
        )
        port map (
            I => ClkIn0P_ik,
            IB => ClkIn0N_ik,
            O => ClkIn0_k
        );

    cTriggerIbufds: IBUFDS
        generic map (
            DIFF_TERM => TRUE
        )
        port map (
            I => TriggerP_i,
            IB => TriggerN_i,
            O => Trigger
        );

    cFfpgCore: entity work.FfpgCore(syn)
        generic map (
            g_ClkFrequency => g_ClkFrequency, -- input clock frequency in Hz
            g_Version => g_Version
        )
        port map (
            -- Wishbone connection
            Clk_ik => Clk_ik,
            Reset_ira => Reset_ira, -- synchronized locally for better timing
            Wb_i => Wb_i,
            Wb_o => Wb_o,
            ---- FMC interface
            -- clock
            ClkIn0_ik => ClkIn0_k,
            -- DACs
            TriggerDac_o => TriggerDac_o,
            VcxoDac_o => VcxoDac_o,
            -- output enable
            Ch1OutputEnable_o => Ch1OutputEnable_o,
            Ch2OutputEnable_o => Ch2OutputEnable_o,
            -- delay configuration
            DelayValue_ob => DelayValue_ob,
            Ch1SetLe_on => Ch1SetLe_on,
            Ch1ResLe_on => Ch1ResLe_on,
            Ch2SetLe_on => Ch2SetLe_on,
            Ch2ResLe_on => Ch2ResLe_on,
            Ch1Set_o => Ch1Set,
            Ch1Res_o => Ch1Res,
            Ch2Set_o => Ch2Set,
            Ch2Res_o => Ch2Res,
            Trigger_i => Trigger,
            -- AD9512 SPI
            SpiAd9512Sclk_o => SpiAd9512Sclk_o,
            SpiAd9512Mosi_o => SpiAd9512Mosi_o,
            SpiAd9512Miso_i => SpiAd9512Miso_i,
            SpiAd9512Cs_on => SpiAd9512Cs_on,
            -- AD9512 func. pin
            Ad9512Func_o => Ad9512Func_o,
            -- clock selection pin of SY58017U
            Clk2Sel_o => Clk2Sel_o,
            -- clock output
            ClkOut_ok => ClkOut_k,
            -- temperature chip interface
            Onewire_io => Onewire_io,
            -- LEDs
            Led_ob => Led_ob
        );

    cCh1SetObufds: OBUFDS
        port map (
            I => Ch1Set,
            O => Ch1SetP_o,
            OB => Ch1SetN_o
        );

    cCh1ResObufds: OBUFDS
        port map (
            I => Ch1Res,
            O => Ch1ResP_o,
            OB => Ch1ResN_o
        );

    cCh2SetObufds: OBUFDS
        port map (
            I => Ch2Set,
            O => Ch2SetP_o,
            OB => Ch2SetN_o
        );

    cCh2ResObufds: OBUFDS
        port map (
            I => Ch2Res,
            O => Ch2ResP_o,
            OB => Ch2ResN_o
        );

    cClkOutOddr: ODDR2
        generic map(
            DDR_ALIGNMENT => "C0",
            SRTYPE => "ASYNC"
        )
        port map (
            Q => ClkOutDdr,
            C0 => ClkOut_k,
            C1 => not ClkOut_k,
            CE => '1',
            D0 => '1',
            D1 => '0',
            R => '0',
            S => '0'
        );

    cClkOutObufds: OBUFDS
        port map (
            I => ClkOutDdr,
            O => ClkOutP_ok,
            OB => ClkOutN_ok
        );

end architecture;
