-------------------------------------------------------------------------------
-- Title      : Top-level entity for implementation on SPEC carrier card
-- Project    : FMC DEL 1ns 2cha (FFPG)
-- URL        : http://www.ohwr.org/projects/fmc-del-1ns-2cha
-------------------------------------------------------------------------------
-- File       : spec_top_ffpg.vhd
-- Author(s)  : Jan Pospisil <j.pospisil@cern.ch>
--              Tom Levens <tom.levens@cern.ch>
-- Company    : CERN (SY-BI-IQ)
-- Created    : 2016-07-26
-- Last update: 2023-02-23
-- Standard   : VHDL'93/02
-------------------------------------------------------------------------------
-- Description: Contains code for one FMC mezzanine, communication via PCIe bus
--              and other board specific code.
-------------------------------------------------------------------------------
-- Copyright (c) 2016 CERN (BE-BI-QP)
--               2023 CERN (SY-BI-IQ)
-------------------------------------------------------------------------------
-- GNU LESSER GENERAL PUBLIC LICENSE
-------------------------------------------------------------------------------
-- This source file is free software; you can redistribute it and/or modify it
-- under the terms of the GNU Lesser General Public License as published by the
-- Free Software Foundation; either version 2.1 of the License, or (at your
-- option) any later version. This source is distributed in the hope that it
-- will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- See the GNU Lesser General Public License for more details. You should have
-- received a copy of the GNU Lesser General Public License along with this
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author          Comment
-- 2016-08-24  1.0      Jan Pospisil    Initial version
-- 2023-02-23  2.0      Tom Levens      Port to new SVEC/SPEC reference designs
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.wishbone_pkg.all;
use work.gencores_pkg.all;
use work.wr_board_pkg.all;
use work.wr_fabric_pkg.all;
use work.FfpgPkg.all;
use work.sourceid_spec_top_ffpg_pkg;

entity spec_top_ffpg is
  generic (
    -- Simulation-mode enable parameter. Set by default (synthesis) to 0, and
    -- changed to non-zero in the instantiation of the top level DUT in the testbench.
    -- Its purpose is to reduce some internal counters/timeouts to speed up simulations.
    g_SIMULATION : integer := 0;
    -- Increase information messages during simulation
    g_VERBOSE    : boolean := False
  );
  port (
    ---------------------------------------------------------------------------
    -- Clocks/resets
    ---------------------------------------------------------------------------

    -- Reset button
    button1_n_i : in std_logic;

    -- 125 MHz PLL reference
    clk_125m_pllref_p_i : in std_logic;
    clk_125m_pllref_n_i : in std_logic;

    -- 20MHz VCXO clock (for WR)
    clk_20m_vcxo_i : in std_logic := '0';

    -- 125 MHz GTP reference
    clk_125m_gtp_n_i : in std_logic := '0';
    clk_125m_gtp_p_i : in std_logic := '0';

    ---------------------------------------------------------------------------
    -- GN4124 interface
    --
    -- gn_gpio_b[1] -> AB19 -> GN4124 GPIO9
    -- gn_gpio_b[0] -> U16  -> GN4124 GPIO8
    ---------------------------------------------------------------------------
    gn_rst_n_i      : in    std_logic;
    gn_p2l_clk_n_i  : in    std_logic;
    gn_p2l_clk_p_i  : in    std_logic;
    gn_p2l_rdy_o    : out   std_logic;
    gn_p2l_dframe_i : in    std_logic;
    gn_p2l_valid_i  : in    std_logic;
    gn_p2l_data_i   : in    std_logic_vector(15 downto 0);
    gn_p_wr_req_i   : in    std_logic_vector(1 downto 0);
    gn_p_wr_rdy_o   : out   std_logic_vector(1 downto 0);
    gn_rx_error_o   : out   std_logic;
    gn_l2p_clk_n_o  : out   std_logic;
    gn_l2p_clk_p_o  : out   std_logic;
    gn_l2p_dframe_o : out   std_logic;
    gn_l2p_valid_o  : out   std_logic;
    gn_l2p_edb_o    : out   std_logic;
    gn_l2p_data_o   : out   std_logic_vector(15 downto 0);
    gn_l2p_rdy_i    : in    std_logic;
    gn_l_wr_rdy_i   : in    std_logic_vector(1 downto 0);
    gn_p_rd_d_rdy_i : in    std_logic_vector(1 downto 0);
    gn_tx_error_i   : in    std_logic;
    gn_vc_rdy_i     : in    std_logic_vector(1 downto 0);
    gn_gpio_b       : inout std_logic_vector(1 downto 0);

    ---------------------------------------------------------------------------
    -- FMC interface
    ---------------------------------------------------------------------------

    -- I2C interface for accessing FMC EEPROM.
    fmc0_scl_b : inout std_logic;
    fmc0_sda_b : inout std_logic;

    -- Presence  (there is a pull-up)
    fmc0_prsnt_m2c_n_i: in std_logic;

    ---------------------------------------------------------------------------
    -- Carrier
    ---------------------------------------------------------------------------

    -- Onewire interface
    onewire_b : inout std_logic;

    ---------------------------------------------------------------------------
    -- Flash memory SPI interface
    ---------------------------------------------------------------------------

    spi_sclk_o : out std_logic;
    spi_ncs_o  : out std_logic;
    spi_mosi_o : out std_logic;
    spi_miso_i : in  std_logic;

    ---------------------------------------------------------------------------
    -- UART
    ---------------------------------------------------------------------------

    uart_rxd_i : in  std_logic := '1';
    uart_txd_o : out std_logic;

    ---------------------------------------------------------------------------
    -- SPI interface to DACs
    ---------------------------------------------------------------------------

    pll25dac_cs_n_o : out std_logic;
    pll20dac_cs_n_o : out std_logic;
    plldac_din_o    : out std_logic;
    plldac_sclk_o   : out std_logic;

    ---------------------------------------------------------------------------
    -- SFP I/O for transceiver
    ---------------------------------------------------------------------------

    sfp_txp_o         : out   std_logic;
    sfp_txn_o         : out   std_logic;
    sfp_rxp_i         : in    std_logic := '0';
    sfp_rxn_i         : in    std_logic := '0';
    sfp_mod_def0_i    : in    std_logic := '0';   -- sfp detect
    sfp_mod_def1_b    : inout std_logic;          -- scl
    sfp_mod_def2_b    : inout std_logic;          -- sda
    sfp_rate_select_o : out   std_logic;
    sfp_tx_fault_i    : in    std_logic := '0';
    sfp_tx_disable_o  : out   std_logic;
    sfp_los_i         : in    std_logic := '0';

    ---------------------------------------------------------------------------
    -- FMC slot 0
    ---------------------------------------------------------------------------

    -- Clock
    Fmc0ClkIn0P_ik        : in    std_logic;
    Fmc0ClkIn0N_ik        : in    std_logic;

    -- DACs
    Fmc0TriggerDac_o      : out   t_Ad5600Interface;
    Fmc0VcxoDac_o         : out   t_Ad5600Interface;

    -- Output enable
    Fmc0Ch1OutputEnable_o : out   std_logic;
    Fmc0Ch2OutputEnable_o : out   std_logic;

    -- Signal feedback
    Fmc0Ch1OutCal_i       : in    std_logic;
    Fmc0Ch2OutCal_i       : in    std_logic;

    -- Delay configuration
    Fmc0DelayValue_ob     : out   unsigned(9 downto 0);
    Fmc0Ch1SetLe_on       : out   std_logic;
    Fmc0Ch1ResLe_on       : out   std_logic;
    Fmc0Ch2SetLe_on       : out   std_logic;
    Fmc0Ch2ResLe_on       : out   std_logic;
    Fmc0Ch1SetP_o         : out   std_logic;
    Fmc0Ch1SetN_o         : out   std_logic;
    Fmc0Ch1ResP_o         : out   std_logic;
    Fmc0Ch1ResN_o         : out   std_logic;
    Fmc0Ch2SetP_o         : out   std_logic;
    Fmc0Ch2SetN_o         : out   std_logic;
    Fmc0Ch2ResP_o         : out   std_logic;
    Fmc0Ch2ResN_o         : out   std_logic;
    Fmc0TriggerP_i        : in    std_logic;
    Fmc0TriggerN_i        : in    std_logic;

    -- AD9512 SPI
    Fmc0SpiAd9512Sclk_o   : out   std_logic;
    Fmc0SpiAd9512Mosi_o   : out   std_logic;
    Fmc0SpiAd9512Miso_i   : in    std_logic;
    Fmc0SpiAd9512Cs_on    : out   std_logic;

    -- AD9512 func. pin
    Fmc0Ad9512Func_o      : out   std_logic;

    -- Clock selection pin of SY58017U
    Fmc0Clk2Sel_o         : out   std_logic;

    -- Clock output
    Fmc0ClkOutP_ok        : out   std_logic;
    Fmc0ClkOutN_ok        : out   std_logic;

    -- Temperature chip interface
    Fmc0Onewire_io        : inout std_logic;

    -- LEDs
    Fmc0Led_ob            : out   std_logic_vector(4 downto 1);

    ---------------------------------------------------------------------------
    -- General
    ---------------------------------------------------------------------------

    -- PCB revision
    pcbrev_i : in std_logic_vector(3 downto 0);

    -- Carrier front panel LEDs
    led_act_o   : out std_logic;
    led_link_o  : out std_logic;

    -- Auxiliary pins
    aux_leds_o : out std_logic_vector(3 downto 0)
  );
end entity spec_top_ffpg;

architecture top of spec_top_ffpg is

  -- Clocks & reset
  signal clk_sys_62m5     : std_logic;
  signal rst_sys_62m5_n   : std_logic := '0';

  -- Wishbone
  signal master_wb_out    : t_wishbone_master_out;
  signal master_wb_in     : t_wishbone_master_in;

  signal metadata_wb_out, ffpg0_wb_out : t_wishbone_slave_out;
  signal metadata_wb_in,  ffpg0_wb_in  : t_wishbone_slave_in;

begin

  cmp_spec_top_ffpg_map : entity work.spec_top_ffpg_map
    port map (
      rst_n_i         => rst_sys_62m5_n,
      clk_i           => clk_sys_62m5,
      wb_i            => master_wb_out,
      wb_o            => master_wb_in,
      metadata_i      => metadata_wb_out,
      metadata_o      => metadata_wb_in,
      ffpg0_i         => ffpg0_wb_out,
      ffpg0_o         => ffpg0_wb_in
    );

  cmp_xwb_metadata : entity work.xwb_metadata
    generic map (
      g_VENDOR_ID     => x"0000_10DC",
      g_DEVICE_ID     => x"465047_02",  -- FPG_02,
      g_VERSION       => sourceid_spec_top_ffpg_pkg.version,
      g_CAPABILITIES  => x"0000_0000",
      g_COMMIT_ID     => sourceid_spec_top_ffpg_pkg.sourceid
    )
    port map (
      clk_i           => clk_sys_62m5,
      rst_n_i         => rst_sys_62m5_n,
      wb_i            => metadata_wb_in,
      wb_o            => metadata_wb_out
    );

  inst_spec_base : entity work.spec_base_wr
    generic map (
      g_WITH_VIC          => FALSE,
      g_WITH_ONEWIRE      => TRUE,
      g_WITH_SPI          => TRUE,
      g_WITH_WR           => FALSE,
      g_WITH_DDR          => FALSE,
      g_APP_OFFSET        => x"0000_4000",
      g_NUM_USER_IRQ      => 0,
      g_SIMULATION        => f_int2bool(g_SIMULATION),
      g_VERBOSE           => g_VERBOSE
    )
    port map (
      clk_125m_pllref_p_i => clk_125m_pllref_p_i,
      clk_125m_pllref_n_i => clk_125m_pllref_n_i,
      clk_20m_vcxo_i      => clk_20m_vcxo_i,
      clk_125m_gtp_n_i    => clk_125m_gtp_n_i,
      clk_125m_gtp_p_i    => clk_125m_gtp_p_i,
      gn_rst_n_i          => gn_rst_n_i,
      gn_p2l_clk_n_i      => gn_p2l_clk_n_i,
      gn_p2l_clk_p_i      => gn_p2l_clk_p_i,
      gn_p2l_rdy_o        => gn_p2l_rdy_o,
      gn_p2l_dframe_i     => gn_p2l_dframe_i,
      gn_p2l_valid_i      => gn_p2l_valid_i,
      gn_p2l_data_i       => gn_p2l_data_i,
      gn_p_wr_req_i       => gn_p_wr_req_i,
      gn_p_wr_rdy_o       => gn_p_wr_rdy_o,
      gn_rx_error_o       => gn_rx_error_o,
      gn_l2p_clk_n_o      => gn_l2p_clk_n_o,
      gn_l2p_clk_p_o      => gn_l2p_clk_p_o,
      gn_l2p_dframe_o     => gn_l2p_dframe_o,
      gn_l2p_valid_o      => gn_l2p_valid_o,
      gn_l2p_edb_o        => gn_l2p_edb_o,
      gn_l2p_data_o       => gn_l2p_data_o,
      gn_l2p_rdy_i        => gn_l2p_rdy_i,
      gn_l_wr_rdy_i       => gn_l_wr_rdy_i,
      gn_p_rd_d_rdy_i     => gn_p_rd_d_rdy_i,
      gn_tx_error_i       => gn_tx_error_i,
      gn_vc_rdy_i         => gn_vc_rdy_i,
      gn_gpio_b           => gn_gpio_b,
      fmc0_scl_b          => fmc0_scl_b,
      fmc0_sda_b          => fmc0_sda_b,
      fmc0_prsnt_m2c_n_i  => fmc0_prsnt_m2c_n_i,
      onewire_b           => onewire_b,
      spi_sclk_o          => spi_sclk_o,
      spi_ncs_o           => spi_ncs_o,
      spi_mosi_o          => spi_mosi_o,
      spi_miso_i          => spi_miso_i,
      pcbrev_i            => pcbrev_i,
      button1_n_i         => button1_n_i,
      uart_rxd_i          => uart_rxd_i,
      uart_txd_o          => uart_txd_o,
      plldac_sclk_o       => plldac_sclk_o,
      plldac_din_o        => plldac_din_o,
      pll25dac_cs_n_o     => pll25dac_cs_n_o,
      pll20dac_cs_n_o     => pll20dac_cs_n_o,
      sfp_txp_o           => sfp_txp_o,
      sfp_txn_o           => sfp_txn_o,
      sfp_rxp_i           => sfp_rxp_i,
      sfp_rxn_i           => sfp_rxn_i,
      sfp_mod_def0_i      => sfp_mod_def0_i,
      sfp_mod_def1_b      => sfp_mod_def1_b,
      sfp_mod_def2_b      => sfp_mod_def2_b,
      sfp_rate_select_o   => sfp_rate_select_o,
      sfp_tx_fault_i      => sfp_tx_fault_i,
      sfp_tx_disable_o    => sfp_tx_disable_o,
      sfp_los_i           => sfp_los_i,
      clk_62m5_sys_o      => clk_sys_62m5,
      rst_62m5_sys_n_o    => rst_sys_62m5_n,
      clk_125m_ref_o      => open,
      rst_125m_ref_n_o    => open,
      irq_user_i          => "",
      app_wb_o            => master_wb_out,
      app_wb_i            => master_wb_in
    );

  ------------------------------------------------------------------------------
  -- Carrier front panel LEDs
  ------------------------------------------------------------------------------

  cmp_vme_access_led : gc_extend_pulse
    generic map (
      g_width     => 2500000
    )
    port map (
      clk_i       => clk_sys_62m5,
      rst_n_i     => rst_sys_62m5_n,
      pulse_i     => master_wb_out.stb,
      extended_o  => led_link_o
    );

  led_act_o <= '0';
  aux_leds_o <= "0000";

  -----------------------------------------------------------------------------
  -- FMC slot 0 - FFPG
  -----------------------------------------------------------------------------

  cFmc0FfpgCore: entity work.FfpgCoreWrapper(sp6)
    generic map (
      g_ClkFrequency    => 62500000,  -- in Hz
      g_Version         => sourceid_spec_top_ffpg_pkg.version
    )
    port map (
      Clk_ik            => clk_sys_62m5,
      Reset_ira         => not rst_sys_62m5_n,
      Wb_i              => ffpg0_wb_in,
      Wb_o              => ffpg0_wb_out,
      ClkIn0P_ik        => Fmc0ClkIn0P_ik,
      ClkIn0N_ik        => Fmc0ClkIn0N_ik,
      TriggerDac_o      => Fmc0TriggerDac_o,
      VcxoDac_o         => Fmc0VcxoDac_o,
      Ch1OutputEnable_o => Fmc0Ch1OutputEnable_o,
      Ch2OutputEnable_o => Fmc0Ch2OutputEnable_o,
      DelayValue_ob     => Fmc0DelayValue_ob,
      Ch1SetLe_on       => Fmc0Ch1SetLe_on,
      Ch1ResLe_on       => Fmc0Ch1ResLe_on,
      Ch2SetLe_on       => Fmc0Ch2SetLe_on,
      Ch2ResLe_on       => Fmc0Ch2ResLe_on,
      Ch1SetP_o         => Fmc0Ch1SetP_o,
      Ch1SetN_o         => Fmc0Ch1SetN_o,
      Ch1ResP_o         => Fmc0Ch1ResP_o,
      Ch1ResN_o         => Fmc0Ch1ResN_o,
      Ch2SetP_o         => Fmc0Ch2SetP_o,
      Ch2SetN_o         => Fmc0Ch2SetN_o,
      Ch2ResP_o         => Fmc0Ch2ResP_o,
      Ch2ResN_o         => Fmc0Ch2ResN_o,
      TriggerP_i        => Fmc0TriggerP_i,
      TriggerN_i        => Fmc0TriggerN_i,
      SpiAd9512Sclk_o   => Fmc0SpiAd9512Sclk_o,
      SpiAd9512Mosi_o   => Fmc0SpiAd9512Mosi_o,
      SpiAd9512Miso_i   => Fmc0SpiAd9512Miso_i,
      SpiAd9512Cs_on    => Fmc0SpiAd9512Cs_on,
      Ad9512Func_o      => Fmc0Ad9512Func_o,
      Clk2Sel_o         => Fmc0Clk2Sel_o,
      ClkOutP_ok        => Fmc0ClkOutP_ok,
      ClkOutN_ok        => Fmc0ClkOutN_ok,
      Onewire_io        => Fmc0Onewire_io,
      Led_ob            => Fmc0Led_ob
    );

end architecture top;
