-------------------------------------------------------------------------------
-- Title      : Top-level entity for implementation on SVEC carrier card
-- Project    : FMC DEL 1ns 2cha (FFPG)
-- URL        : http://www.ohwr.org/projects/fmc-del-1ns-2cha
-------------------------------------------------------------------------------
-- File       : svec_top_ffpg.vhd
-- Author(s)  : Jan Pospisil <j.pospisil@cern.ch>
--              Tom Levens <tom.levens@cern.ch>
-- Company    : CERN (SY-BI-IQ)
-- Created    : 2016-07-26
-- Last update: 2023-02-23
-- Standard   : VHDL'93/02
-------------------------------------------------------------------------------
-- Description: Contains code for two FMC mezzanines, communication via VME bus
--              and other board specific code.
-------------------------------------------------------------------------------
-- Copyright (c) 2016 CERN (BE-BI-QP)
--               2023 CERN (SY-BI-IQ)
-------------------------------------------------------------------------------
-- GNU LESSER GENERAL PUBLIC LICENSE
-------------------------------------------------------------------------------
-- This source file is free software; you can redistribute it and/or modify it
-- under the terms of the GNU Lesser General Public License as published by the
-- Free Software Foundation; either version 2.1 of the License, or (at your
-- option) any later version. This source is distributed in the hope that it
-- will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- See the GNU Lesser General Public License for more details. You should have
-- received a copy of the GNU Lesser General Public License along with this
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author          Comment
-- 2016-08-24  1.0      Jan Pospisil    Initial version
-- 2023-02-23  2.0      Tom Levens      Port to new SVEC/SPEC reference designs
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.wishbone_pkg.all;
use work.gencores_pkg.all;
use work.wr_board_pkg.all;
use work.wr_fabric_pkg.all;
use work.FfpgPkg.all;
use work.sourceid_svec_top_ffpg_pkg;

entity svec_top_ffpg is
  generic (
    -- Simulation-mode enable parameter. Set by default (synthesis) to 0, and
    -- changed to non-zero in the instantiation of the top level DUT in the testbench.
    -- Its purpose is to reduce some internal counters/timeouts to speed up simulations.
    g_SIMULATION : integer := 0;
    -- Increase information messages during simulation
    g_VERBOSE    : boolean := False
  );
  port (
    ---------------------------------------------------------------------------
    -- Clocks/resets
    ---------------------------------------------------------------------------

    -- Reset from system fpga
    rst_n_i : in std_logic;

    -- 125 MHz PLL reference
    clk_125m_pllref_p_i : in std_logic;
    clk_125m_pllref_n_i : in std_logic;

    -- 20MHz VCXO clock (for WR)
    clk_20m_vcxo_i : in std_logic := '0';

    -- 125 MHz GTP reference
    clk_125m_gtp_n_i : in std_logic := '0';
    clk_125m_gtp_p_i : in std_logic := '0';

    ---------------------------------------------------------------------------
    -- VME interface
    ---------------------------------------------------------------------------

    vme_write_n_i    : in    std_logic;
    vme_sysreset_n_i : in    std_logic;
    vme_retry_oe_o   : out   std_logic;
    vme_retry_n_o    : out   std_logic;
    vme_lword_n_b    : inout std_logic;
    vme_iackout_n_o  : out   std_logic;
    vme_iackin_n_i   : in    std_logic;
    vme_iack_n_i     : in    std_logic;
    vme_gap_i        : in    std_logic;
    vme_dtack_oe_o   : out   std_logic;
    vme_dtack_n_o    : out   std_logic;
    vme_ds_n_i       : in    std_logic_vector(1 downto 0);
    vme_data_oe_n_o  : out   std_logic;
    vme_data_dir_o   : out   std_logic;
    vme_berr_o       : out   std_logic;
    vme_as_n_i       : in    std_logic;
    vme_addr_oe_n_o  : out   std_logic;
    vme_addr_dir_o   : out   std_logic;
    vme_irq_o        : out   std_logic_vector(7 downto 1);
    vme_ga_i         : in    std_logic_vector(4 downto 0);
    vme_data_b       : inout std_logic_vector(31 downto 0);
    vme_am_i         : in    std_logic_vector(5 downto 0);
    vme_addr_b       : inout std_logic_vector(31 downto 1);

    ---------------------------------------------------------------------------
    -- FMC interface
    ---------------------------------------------------------------------------

    -- I2C interface for accessing FMC EEPROM.
    fmc0_scl_b : inout std_logic;
    fmc0_sda_b : inout std_logic;
    fmc1_scl_b : inout std_logic;
    fmc1_sda_b : inout std_logic;

    -- Presence  (there is a pull-up)
    fmc0_prsnt_m2c_n_i: in std_logic;
    fmc1_prsnt_m2c_n_i: in std_logic;

    ---------------------------------------------------------------------------
    -- Carrier
    ---------------------------------------------------------------------------

    -- Onewire interface
    onewire_b : inout std_logic;

    -- Carrier I2C eeprom
    carrier_scl_b : inout std_logic;
    carrier_sda_b : inout std_logic;

    ---------------------------------------------------------------------------
    -- Flash memory SPI interface
    ---------------------------------------------------------------------------

    spi_sclk_o : out std_logic;
    spi_ncs_o  : out std_logic;
    spi_mosi_o : out std_logic;
    spi_miso_i : in  std_logic;

    ---------------------------------------------------------------------------
    -- UART
    ---------------------------------------------------------------------------

    uart_rxd_i : in  std_logic := '1';
    uart_txd_o : out std_logic;

    ---------------------------------------------------------------------------
    -- SPI interface to DACs
    ---------------------------------------------------------------------------

    plldac_sclk_o     : out std_logic;
    plldac_din_o      : out std_logic;
    pll20dac_din_o    : out std_logic;
    pll20dac_sclk_o   : out std_logic;
    pll20dac_sync_n_o : out std_logic;
    pll25dac_din_o    : out std_logic;
    pll25dac_sclk_o   : out std_logic;
    pll25dac_sync_n_o : out std_logic;

    ---------------------------------------------------------------------------
    -- SFP I/O for transceiver
    ---------------------------------------------------------------------------

    sfp_txp_o         : out   std_logic;
    sfp_txn_o         : out   std_logic;
    sfp_rxp_i         : in    std_logic := '0';
    sfp_rxn_i         : in    std_logic := '0';
    sfp_mod_def0_i    : in    std_logic := '0';   -- sfp detect
    sfp_mod_def1_b    : inout std_logic;          -- scl
    sfp_mod_def2_b    : inout std_logic;          -- sda
    sfp_rate_select_o : out   std_logic;
    sfp_tx_fault_i    : in    std_logic := '0';
    sfp_tx_disable_o  : out   std_logic;
    sfp_los_i         : in    std_logic := '0';

    ---------------------------------------------------------------------------
    -- FMC slot 0
    ---------------------------------------------------------------------------

    -- Clock
    Fmc0ClkIn0P_ik        : in    std_logic;
    Fmc0ClkIn0N_ik        : in    std_logic;

    -- DACs
    Fmc0TriggerDac_o      : out   t_Ad5600Interface;
    Fmc0VcxoDac_o         : out   t_Ad5600Interface;

    -- Output enable
    Fmc0Ch1OutputEnable_o : out   std_logic;
    Fmc0Ch2OutputEnable_o : out   std_logic;

    -- Signal feedback
    Fmc0Ch1OutCal_i       : in    std_logic;
    Fmc0Ch2OutCal_i       : in    std_logic;

    -- Delay configuration
    Fmc0DelayValue_ob     : out   unsigned(9 downto 0);
    Fmc0Ch1SetLe_on       : out   std_logic;
    Fmc0Ch1ResLe_on       : out   std_logic;
    Fmc0Ch2SetLe_on       : out   std_logic;
    Fmc0Ch2ResLe_on       : out   std_logic;
    Fmc0Ch1SetP_o         : out   std_logic;
    Fmc0Ch1SetN_o         : out   std_logic;
    Fmc0Ch1ResP_o         : out   std_logic;
    Fmc0Ch1ResN_o         : out   std_logic;
    Fmc0Ch2SetP_o         : out   std_logic;
    Fmc0Ch2SetN_o         : out   std_logic;
    Fmc0Ch2ResP_o         : out   std_logic;
    Fmc0Ch2ResN_o         : out   std_logic;
    Fmc0TriggerP_i        : in    std_logic;
    Fmc0TriggerN_i        : in    std_logic;

    -- AD9512 SPI
    Fmc0SpiAd9512Sclk_o   : out   std_logic;
    Fmc0SpiAd9512Mosi_o   : out   std_logic;
    Fmc0SpiAd9512Miso_i   : in    std_logic;
    Fmc0SpiAd9512Cs_on    : out   std_logic;

    -- AD9512 func. pin
    Fmc0Ad9512Func_o      : out   std_logic;

    -- Clock selection pin of SY58017U
    Fmc0Clk2Sel_o         : out   std_logic;

    -- Clock output
    Fmc0ClkOutP_ok        : out   std_logic;
    Fmc0ClkOutN_ok        : out   std_logic;

    -- Temperature chip interface
    Fmc0Onewire_io        : inout std_logic;

    -- LEDs
    Fmc0Led_ob            : out   std_logic_vector(4 downto 1);

    ---------------------------------------------------------------------------
    -- FMC slot 1
    ---------------------------------------------------------------------------

    -- Clock
    Fmc1ClkIn0P_ik        : in    std_logic;
    Fmc1ClkIn0N_ik        : in    std_logic;

    -- DACs
    Fmc1TriggerDac_o      : out   t_Ad5600Interface;
    Fmc1VcxoDac_o         : out   t_Ad5600Interface;

    -- Output enable
    Fmc1Ch1OutputEnable_o : out   std_logic;
    Fmc1Ch2OutputEnable_o : out   std_logic;

    -- Signal feedback
    Fmc1Ch1OutCal_i       : in    std_logic;
    Fmc1Ch2OutCal_i       : in    std_logic;

    -- Delay configuration
    Fmc1DelayValue_ob     : out   unsigned(9 downto 0);
    Fmc1Ch1SetLe_on       : out   std_logic;
    Fmc1Ch1ResLe_on       : out   std_logic;
    Fmc1Ch2SetLe_on       : out   std_logic;
    Fmc1Ch2ResLe_on       : out   std_logic;
    Fmc1Ch1SetP_o         : out   std_logic;
    Fmc1Ch1SetN_o         : out   std_logic;
    Fmc1Ch1ResP_o         : out   std_logic;
    Fmc1Ch1ResN_o         : out   std_logic;
    Fmc1Ch2SetP_o         : out   std_logic;
    Fmc1Ch2SetN_o         : out   std_logic;
    Fmc1Ch2ResP_o         : out   std_logic;
    Fmc1Ch2ResN_o         : out   std_logic;
    Fmc1TriggerP_i        : in    std_logic;
    Fmc1TriggerN_i        : in    std_logic;

    -- AD9512 SPI
    Fmc1SpiAd9512Sclk_o   : out   std_logic;
    Fmc1SpiAd9512Mosi_o   : out   std_logic;
    Fmc1SpiAd9512Miso_i   : in    std_logic;
    Fmc1SpiAd9512Cs_on    : out   std_logic;

    -- AD9512 func. pin
    Fmc1Ad9512Func_o      : out   std_logic;

    -- Clock selection pin of SY58017U
    Fmc1Clk2Sel_o         : out   std_logic;

    -- Clock output
    Fmc1ClkOutP_ok        : out   std_logic;
    Fmc1ClkOutN_ok        : out   std_logic;

    -- Temperature chip interface
    Fmc1Onewire_io        : inout std_logic;

    -- LEDs
    Fmc1Led_ob            : out   std_logic_vector(4 downto 1);

    ---------------------------------------------------------------------------
    -- General
    ---------------------------------------------------------------------------

    -- PCB revision
    pcbrev_i : in std_logic_vector(4 downto 0);

    -- Carrier front panel LEDs
    fp_led_line_oen_o : out std_logic_vector(1 downto 0);
    fp_led_line_o     : out std_logic_vector(1 downto 0);
    fp_led_column_o   : out std_logic_vector(3 downto 0);

    -- GPIO
    fp_gpio1_b      : in  std_logic;
    fp_gpio2_b      : in  std_logic;
    fp_gpio3_b      : in  std_logic;
    fp_gpio4_b      : in  std_logic;
    fp_term_en_o    : out std_logic_vector(4 downto 1);
    fp_gpio1_a2b_o  : out std_logic;
    fp_gpio2_a2b_o  : out std_logic;
    fp_gpio34_a2b_o : out std_logic
  );
end entity svec_top_ffpg;

architecture top of svec_top_ffpg is

  -- Clocks & reset
  signal clk_sys_62m5     : std_logic;
  signal rst_sys_62m5_n   : std_logic := '0';

  -- Wishbone
  signal master_wb_out    : t_wishbone_master_out;
  signal master_wb_in     : t_wishbone_master_in;

  signal metadata_wb_out, ffpg0_wb_out, ffpg1_wb_out : t_wishbone_slave_out;
  signal metadata_wb_in,  ffpg0_wb_in,  ffpg1_wb_in  : t_wishbone_slave_in;

  -- Front panel LED control
  signal led_state      : std_logic_vector(15 downto 0);
  signal vme_access     : std_logic;

  signal fp_gpio_in     : std_logic_vector(4 downto 1);
  signal fp_gpio_sync   : std_logic_vector(4 downto 1);
  signal fp_gpio        : std_logic_vector(4 downto 1);
  signal fp_gpio_led    : std_logic_vector(4 downto 1);

begin

  cmp_svec_top_ffpg_map : entity work.svec_top_ffpg_map
    port map (
      rst_n_i         => rst_sys_62m5_n,
      clk_i           => clk_sys_62m5,
      wb_i            => master_wb_out,
      wb_o            => master_wb_in,
      metadata_i      => metadata_wb_out,
      metadata_o      => metadata_wb_in,
      ffpg0_i         => ffpg0_wb_out,
      ffpg0_o         => ffpg0_wb_in,
      ffpg1_i         => ffpg1_wb_out,
      ffpg1_o         => ffpg1_wb_in
    );

  cmp_xwb_metadata : entity work.xwb_metadata
    generic map (
      g_VENDOR_ID     => x"0000_10DC",
      g_DEVICE_ID     => x"465047_01",  -- FPG_01,
      g_VERSION       => sourceid_svec_top_ffpg_pkg.version,
      g_CAPABILITIES  => x"0000_0000",
      g_COMMIT_ID     => sourceid_svec_top_ffpg_pkg.sourceid
    )
    port map (
      clk_i           => clk_sys_62m5,
      rst_n_i         => rst_sys_62m5_n,
      wb_i            => metadata_wb_in,
      wb_o            => metadata_wb_out
    );

  inst_svec_base: entity work.svec_base_wr
    generic map (
      g_WITH_VIC            => FALSE,
      g_WITH_ONEWIRE        => TRUE,
      g_WITH_SPI            => TRUE,
      g_WITH_WR             => FALSE,
      g_WITH_DDR4           => FALSE,
      g_WITH_DDR5           => FALSE,
      g_APP_OFFSET          => x"0000_4000",
      g_NUM_USER_IRQ        => 0,
      g_SIMULATION          => g_SIMULATION,
      g_VERBOSE             => g_VERBOSE
    )
    port map (
      rst_n_i               => rst_n_i,
      clk_125m_pllref_p_i   => clk_125m_pllref_p_i,
      clk_125m_pllref_n_i   => clk_125m_pllref_n_i,
      clk_20m_vcxo_i        => clk_20m_vcxo_i,
      clk_125m_gtp_n_i      => clk_125m_gtp_n_i,
      clk_125m_gtp_p_i      => clk_125m_gtp_p_i,
      vme_write_n_i         => vme_write_n_i,
      vme_sysreset_n_i      => vme_sysreset_n_i,
      vme_retry_oe_o        => vme_retry_oe_o,
      vme_retry_n_o         => vme_retry_n_o,
      vme_lword_n_b         => vme_lword_n_b,
      vme_iackout_n_o       => vme_iackout_n_o,
      vme_iackin_n_i        => vme_iackin_n_i,
      vme_iack_n_i          => vme_iack_n_i,
      vme_gap_i             => vme_gap_i,
      vme_dtack_oe_o        => vme_dtack_oe_o,
      vme_dtack_n_o         => vme_dtack_n_o,
      vme_ds_n_i            => vme_ds_n_i,
      vme_data_oe_n_o       => vme_data_oe_n_o,
      vme_data_dir_o        => vme_data_dir_o,
      vme_berr_o            => vme_berr_o,
      vme_as_n_i            => vme_as_n_i,
      vme_addr_oe_n_o       => vme_addr_oe_n_o,
      vme_addr_dir_o        => vme_addr_dir_o,
      vme_irq_o             => vme_irq_o,
      vme_ga_i              => vme_ga_i,
      vme_data_b            => vme_data_b,
      vme_am_i              => vme_am_i,
      vme_addr_b            => vme_addr_b,
      fmc0_scl_b            => fmc0_scl_b,
      fmc0_sda_b            => fmc0_sda_b,
      fmc1_scl_b            => fmc1_scl_b,
      fmc1_sda_b            => fmc1_sda_b,
      fmc0_prsnt_m2c_n_i    => fmc0_prsnt_m2c_n_i,
      fmc1_prsnt_m2c_n_i    => fmc1_prsnt_m2c_n_i,
      onewire_b             => onewire_b,
      carrier_scl_b         => carrier_scl_b,
      carrier_sda_b         => carrier_sda_b,
      spi_sclk_o            => spi_sclk_o,
      spi_ncs_o             => spi_ncs_o,
      spi_mosi_o            => spi_mosi_o,
      spi_miso_i            => spi_miso_i,
      uart_rxd_i            => uart_rxd_i,
      uart_txd_o            => uart_txd_o,
      plldac_sclk_o         => plldac_sclk_o,
      plldac_din_o          => plldac_din_o,
      pll20dac_din_o        => pll20dac_din_o,
      pll20dac_sclk_o       => pll20dac_sclk_o,
      pll20dac_sync_n_o     => pll20dac_sync_n_o,
      pll25dac_din_o        => pll25dac_din_o,
      pll25dac_sclk_o       => pll25dac_sclk_o,
      pll25dac_sync_n_o     => pll25dac_sync_n_o,
      sfp_txp_o             => sfp_txp_o,
      sfp_txn_o             => sfp_txn_o,
      sfp_rxp_i             => sfp_rxp_i,
      sfp_rxn_i             => sfp_rxn_i,
      sfp_mod_def0_i        => sfp_mod_def0_i,
      sfp_mod_def1_b        => sfp_mod_def1_b,
      sfp_mod_def2_b        => sfp_mod_def2_b,
      sfp_rate_select_o     => sfp_rate_select_o,
      sfp_tx_fault_i        => sfp_tx_fault_i,
      sfp_tx_disable_o      => sfp_tx_disable_o,
      sfp_los_i             => sfp_los_i,
      pcbrev_i              => pcbrev_i,
      clk_sys_62m5_o        => clk_sys_62m5,
      rst_sys_62m5_n_o      => rst_sys_62m5_n,
      clk_ref_125m_o        => open,
      rst_ref_125m_n_o      => open,
      irq_user_i            => "",
      app_wb_o              => master_wb_out,
      app_wb_i              => master_wb_in
    );

  ------------------------------------------------------------------------------
  -- Carrier front panel LEDs and LEMOs
  ------------------------------------------------------------------------------

  cmp_led_controller : entity work.gc_bicolor_led_ctrl
    generic map (
      g_nb_column     => 4,
      g_nb_line       => 2,
      g_clk_freq      => 62500000,    -- in Hz
      g_refresh_rate  => 250          -- in Hz
    )
    port map (
      rst_n_i         => rst_sys_62m5_n,
      clk_i           => clk_sys_62m5,
      led_intensity_i => "1100100",  -- in %
      led_state_i     => led_state,
      column_o        => fp_led_column_o,
      line_o          => fp_led_line_o,
      line_oen_o      => fp_led_line_oen_o
    );

  cmp_vme_access_led : gc_extend_pulse
    generic map (
      g_width     => 2500000
    )
    port map (
      clk_i       => clk_sys_62m5,
      rst_n_i     => rst_sys_62m5_n,
      pulse_i     => master_wb_out.stb,
      extended_o  => vme_access
    );

  -- LED order on front panel (top to bottom)
  --  7-6    5-4    3-2   1-0
  -- 15-14  13-12  11-10  9-8
  led_state(1 downto 0)   <= c_led_green when vme_access = '1' else c_led_off;
  led_state(3 downto 2)   <= c_led_off;
  led_state(5 downto 4)   <= c_led_off;
  led_state(7 downto 6)   <= c_led_off;
  led_state(9 downto 8)   <= c_led_green when fp_gpio_led(4) = '1' else c_led_off;
  led_state(11 downto 10) <= c_led_green when fp_gpio_led(3) = '1' else c_led_off;
  led_state(13 downto 12) <= c_led_green when fp_gpio_led(2) = '1' else c_led_off;
  led_state(15 downto 14) <= c_led_green when fp_gpio_led(1) = '1' else c_led_off;

  -- Front panel IO configuration
  fp_term_en_o    <= (others => '0');
  fp_gpio1_a2b_o  <= '0';
  fp_gpio2_a2b_o  <= '0';
  fp_gpio34_a2b_o <= '0';

  fp_gpio_in(1) <= fp_gpio1_b;
  fp_gpio_in(2) <= fp_gpio2_b;
  fp_gpio_in(3) <= fp_gpio3_b;
  fp_gpio_in(4) <= fp_gpio4_b;

  -- TODO: I put an extra gc_sync_register here to keep ISE happy as the
  -- SVEC base design has constraints for it. This can be removed once
  -- this issue is resolved:
  --   https://gitlab.cern.ch/be-cem-edl/fec/hardware-modules/svec/-/issues/25
  cmp_fp_gpio_sync_register: gc_sync_register
    generic map (
      g_width   => 4
    )
    port map (
      clk_i     => clk_sys_62m5,
      rst_n_a_i => rst_sys_62m5_n,
      d_i       => fp_gpio_in,
      q_o       => fp_gpio_sync
    );

  -- Front panel IO synchronisation
  gen_fp_gpio_sync: for i in 1 to 4 generate
    begin
      cmp_fp_gpio_sync_ffs: gc_sync_ffs
        port map (
          clk_i    => clk_sys_62m5,
          rst_n_i  => '1',
          data_i   => fp_gpio_sync(i),
          ppulse_o => fp_gpio(i)
        );

      cmp_fp_gpio_led : gc_extend_pulse
        generic map (
          g_width    => 2500000
        )
        port map (
          clk_i      => clk_sys_62m5,
          rst_n_i    => rst_sys_62m5_n,
          pulse_i    => fp_gpio(i),
          extended_o => fp_gpio_led(i)
        );
    end generate;

  -----------------------------------------------------------------------------
  -- FMC slot 0 - FFPG
  -----------------------------------------------------------------------------

  cFmc0FfpgCore: entity work.FfpgCoreWrapper(sp6)
    generic map (
      g_ClkFrequency    => 62500000,  -- in Hz
      g_Version         => sourceid_svec_top_ffpg_pkg.version
    )
    port map (
      Clk_ik            => clk_sys_62m5,
      Reset_ira         => not rst_sys_62m5_n,
      Wb_i              => ffpg0_wb_in,
      Wb_o              => ffpg0_wb_out,
      ClkIn0P_ik        => Fmc0ClkIn0P_ik,
      ClkIn0N_ik        => Fmc0ClkIn0N_ik,
      TriggerDac_o      => Fmc0TriggerDac_o,
      VcxoDac_o         => Fmc0VcxoDac_o,
      Ch1OutputEnable_o => Fmc0Ch1OutputEnable_o,
      Ch2OutputEnable_o => Fmc0Ch2OutputEnable_o,
      DelayValue_ob     => Fmc0DelayValue_ob,
      Ch1SetLe_on       => Fmc0Ch1SetLe_on,
      Ch1ResLe_on       => Fmc0Ch1ResLe_on,
      Ch2SetLe_on       => Fmc0Ch2SetLe_on,
      Ch2ResLe_on       => Fmc0Ch2ResLe_on,
      Ch1SetP_o         => Fmc0Ch1SetP_o,
      Ch1SetN_o         => Fmc0Ch1SetN_o,
      Ch1ResP_o         => Fmc0Ch1ResP_o,
      Ch1ResN_o         => Fmc0Ch1ResN_o,
      Ch2SetP_o         => Fmc0Ch2SetP_o,
      Ch2SetN_o         => Fmc0Ch2SetN_o,
      Ch2ResP_o         => Fmc0Ch2ResP_o,
      Ch2ResN_o         => Fmc0Ch2ResN_o,
      TriggerP_i        => Fmc0TriggerP_i,
      TriggerN_i        => Fmc0TriggerN_i,
      SpiAd9512Sclk_o   => Fmc0SpiAd9512Sclk_o,
      SpiAd9512Mosi_o   => Fmc0SpiAd9512Mosi_o,
      SpiAd9512Miso_i   => Fmc0SpiAd9512Miso_i,
      SpiAd9512Cs_on    => Fmc0SpiAd9512Cs_on,
      Ad9512Func_o      => Fmc0Ad9512Func_o,
      Clk2Sel_o         => Fmc0Clk2Sel_o,
      ClkOutP_ok        => Fmc0ClkOutP_ok,
      ClkOutN_ok        => Fmc0ClkOutN_ok,
      Onewire_io        => Fmc0Onewire_io,
      Led_ob            => Fmc0Led_ob
    );

  -----------------------------------------------------------------------------
  -- FMC slot 1 - FFPG
  -----------------------------------------------------------------------------

  cFmc1FfpgCore: entity work.FfpgCoreWrapper(sp6)
    generic map (
      g_ClkFrequency    => 62500000,  -- in Hz
      g_Version         => sourceid_svec_top_ffpg_pkg.version
    )
    port map (
      Clk_ik            => clk_sys_62m5,
      Reset_ira         => not rst_sys_62m5_n,
      Wb_i              => ffpg1_wb_in,
      Wb_o              => ffpg1_wb_out,
      ClkIn0P_ik        => Fmc1ClkIn0P_ik,
      ClkIn0N_ik        => Fmc1ClkIn0N_ik,
      TriggerDac_o      => Fmc1TriggerDac_o,
      VcxoDac_o         => Fmc1VcxoDac_o,
      Ch1OutputEnable_o => Fmc1Ch1OutputEnable_o,
      Ch2OutputEnable_o => Fmc1Ch2OutputEnable_o,
      DelayValue_ob     => Fmc1DelayValue_ob,
      Ch1SetLe_on       => Fmc1Ch1SetLe_on,
      Ch1ResLe_on       => Fmc1Ch1ResLe_on,
      Ch2SetLe_on       => Fmc1Ch2SetLe_on,
      Ch2ResLe_on       => Fmc1Ch2ResLe_on,
      Ch1SetP_o         => Fmc1Ch1SetP_o,
      Ch1SetN_o         => Fmc1Ch1SetN_o,
      Ch1ResP_o         => Fmc1Ch1ResP_o,
      Ch1ResN_o         => Fmc1Ch1ResN_o,
      Ch2SetP_o         => Fmc1Ch2SetP_o,
      Ch2SetN_o         => Fmc1Ch2SetN_o,
      Ch2ResP_o         => Fmc1Ch2ResP_o,
      Ch2ResN_o         => Fmc1Ch2ResN_o,
      TriggerP_i        => Fmc1TriggerP_i,
      TriggerN_i        => Fmc1TriggerN_i,
      SpiAd9512Sclk_o   => Fmc1SpiAd9512Sclk_o,
      SpiAd9512Mosi_o   => Fmc1SpiAd9512Mosi_o,
      SpiAd9512Miso_i   => Fmc1SpiAd9512Miso_i,
      SpiAd9512Cs_on    => Fmc1SpiAd9512Cs_on,
      Ad9512Func_o      => Fmc1Ad9512Func_o,
      Clk2Sel_o         => Fmc1Clk2Sel_o,
      ClkOutP_ok        => Fmc1ClkOutP_ok,
      ClkOutN_ok        => Fmc1ClkOutN_ok,
      Onewire_io        => Fmc1Onewire_io,
      Led_ob            => Fmc1Led_ob
    );

end architecture top;
